#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# ***** BEGIN LICENSE BLOCK *****
# Copyright (C) 2012-2013  Hayaki Saito <user@zuse.jp>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ***** END LICENSE BLOCK *****

__author__ = "Hayaki Saito (user@zuse.jp)"
__version__ = "0.1.2"
__license__ = "GPL v3"

import os
import sys
import optparse
import select
import logging
from drcs import DrcsWriter


def _printver():
        print '''
drcsconv %s
Copyright (C) 2012-2014 Hayaki Saito <user@zuse.jp>.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see http://www.gnu.org/licenses/.
        ''' % __version__


def _parse_args():

    parser = optparse.OptionParser()

    parser.add_option("-8", "--8bit-mode",
                      default=False,
                      action="store_true",
                      dest="f8bit",
                      help="Generate a drcs font image for 8bit terminal"
                           " or printer")

    parser.add_option("-7", "--7bit-mode",
                      action="store_false",
                      dest="f8bit",
                      help="Generate a drcs font image for 7bit terminal"
                           " or printer")

    parser.add_option("-n", "--negate",
                      action="store_true",
                      dest="negate",
                      default=False,
                      help="Negate image")

    parser.add_option("-u", "--unicode",
                      action="store_true",
                      dest="use_unicode",
                      default=False,
                      help="Use drcsterm's DRCS-unicode mapping")

    parser.add_option("-c", "--columns",
                      dest="columns",
                      default="62",
                      help="Image width in cell size (default=62, max=62)")

    parser.add_option("-r", "--rows",
                      dest="rows",
                      help="Image height in cell size")

    parser.add_option("-t", "--text",
                      action="store_true",
                      dest="text",
                      default=False,
                      help="Interpret input stream as a text run")

    parser.add_option("-f", "--font",
                      dest="font",
                      help="Specify the absolute path of font file if"
                           " -t option is set")

    parser.add_option("--ncolor",
                      dest="ncolor",
                      action="store",
                      type="int",
                      default=1,
                      help="Specify number of color")

    parser.add_option('--version', dest='version',
                      action="store_true", default=False,
                      help='show version')

    return parser.parse_args()


def _drawtext(options, args):

    if select.select([sys.stdin, ], [], [], 0.0):
        text = sys.stdin.read()
    elif len(args) == 0 or args[0] == '-':
        text = sys.stdin.read()

    import re
    text = re.sub('[\x00-\x1f\x7f]', '', text)

    text = unicode(text, "utf-8", "ignore")
    from PIL import Image
    from PIL import ImageDraw
    from PIL import ImageFont

    if options.font:
        fontfile = options.font
    else:
        import inspect
        name = "unifont-5.1.20080907.ttf"
        filename = inspect.getfile(inspect.currentframe())
        dirpath = os.path.abspath(os.path.dirname(filename))
        fontfile = os.path.join(dirpath, name)

    font = ImageFont.truetype(filename=fontfile,
                              size=50)
    w, h = font.getsize(text)
    image = Image.new('RGB', (w, h + 2), (255, 255, 255))
    draw = ImageDraw.Draw(image)
    draw.text((0, 0), text, font=font, fill=(0, 0, 0))
    import wcwidth
    columns = wcwidth.wcswidth(text)
    rows = 1

    writer = DrcsWriter(f8bit=options.f8bit)
    writer.draw(image,
                columns,
                rows,
                options.negate,
                options.use_unicode,
                ncolor=options.ncolor)

def _filenize(f):
    import stat

    mode = os.fstat(f.fileno()).st_mode
    if stat.S_ISFIFO(mode) or os.isatty(f.fileno()):
        try:
            from cStringIO import StringIO
        except ImportError:
            from StringIO import StringIO
        return StringIO(f.read())
    return f


def _drawimage(options, args):

    columns = int(options.columns)
    if columns > 62:
        message = "Wrong columns value is specified (max: 62)."
        raise optparse.OptionValueError(message)

    if select.select([sys.stdin, ], [], [], 0.0):
        imagefile = _filenize(sys.stdin)
    elif len(args) == 0 or args[0] == '-':
        imagefile = _filenize(sys.stdin)
    else:
        imagefile = args[0]

    from PIL import Image  # PIL
    image = Image.open(imagefile)

    rows = options.rows
    if not rows is None:
        rows = int(rows)

    output = sys.stdout
    output.write("\x1b[?8800h")

    writer = DrcsWriter(f8bit=options.f8bit)
    writer.draw(image,
                columns,
                rows,
                options.negate,
                options.use_unicode,
                output=output,
                ncolor=options.ncolor)


def _mainimpl():

    rcdir = os.path.join(os.getenv("HOME"), ".pydrcs")
    logdir = os.path.join(rcdir, "log")
    if not os.path.exists(logdir):
        os.makedirs(logdir)

    logfile = os.path.join(logdir, "log.txt")
    logging.basicConfig(filename=logfile, filemode="w")

    try:
        options, args = _parse_args()

        if options.version:
            _printver()
            return

        if options.text:
            _drawtext(options, args)
        else:
            _drawimage(options, args)
    except optparse.OptionValueError, e:
        logging.exception(e)
        print e
    except Exception, e:
        logging.exception(e)
        print e

def main():
    _mainimpl()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# ***** BEGIN LICENSE BLOCK *****
# Copyright (C) 2012-2013  Hayaki Saito <user@zuse.jp>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ***** END LICENSE BLOCK *****

import sys
import os

class DrcsConverter:

    def __init__(self, image, f8bit, columns,
                 rows=None, negate=False, use_unicode=False, ncolor=1):
        if f8bit:  # 8bit mode
            self.DCS = '\x90'
            self.ST = '\x9c'
        else:
            self.DCS = '\x1bP'
            self.ST = '\x1b\\'

        self.cellwidth = 15
        self.cellheight = 24
        self.columns = columns
        width, height = image.size
        if rows is None:
            display_width = columns * self.cellwidth
            display_height = 1.0 * display_width * height / width
            rows = int(display_height / self.cellheight)
        self.rows = rows
        self.negate = negate
        self._use_unicode = use_unicode
        width = self.cellwidth * self.columns
        height = self.cellheight * rows
        image = image.resize((width, height))

        if ncolor > 256:
            ncolor = 256
        self._ncolor = ncolor

        if ncolor == 1:
            image = image.convert("L")
            image = image.convert("1")
            self.palette = None
        else:
            from PIL import Image
            image = image.convert("P", palette=Image.ADAPTIVE, colors=ncolor)
            self.palette = image.getpalette()
        self.data = image.getdata()

    def __write_header(self, output, fbyte):

        # start Device Control String (DCS)
        output.write(self.DCS)

        # write header
        cellwidth = self.cellwidth
        cellheight = self.cellheight
        if self._ncolor > 1:
            pt = 3
        else:
            pt = 1
        output.write('1;0;0;%d;1;%d;%d;0{ %c' % (cellwidth, pt, cellheight, fbyte))

    def _write_sixel_palette(self, output):

        palette = self.palette

        # write palette section
        for i in xrange(0, self._ncolor * 3, 3):
            no = i / 3
            r = palette[i + 0] * 100 / 256
            g = palette[i + 1] * 100 / 256
            b = palette[i + 2] * 100 / 256
            output.write('#%d;2;%d;%d;%d' % (no, r, g, b))

    def _write_colored_sixel(self, output, data, width, top, keycolor):

        n = 1
        for y in xrange(top, top + 6):
            p = y * width
            cached_no = data[p]
            count = 1
            c = -1
            for x in xrange(0, width):
                color_no = data[p + x]
                if color_no == cached_no and count < 255:
                    count += 1
                else:
                    if cached_no == keycolor:
                        c = 0x3f
                    else:
                        c = n + 0x3f
                    if count == 1:
                        output.write('#%d%c' % (cached_no, c))
                    elif count == 2:
                        output.write('#%d%c%c' % (cached_no, c, c))
                        count = 1
                    else:
                        output.write('#%d!%d%c' % (cached_no, count, c))
                        count = 1
                    cached_no = color_no
            if c != -1:
                if cached_no == keycolor:
                    c = 0x3f 
                if count == 1:
                    output.write('#%d%c' % (cached_no, c))
                elif count == 2:
                    output.write('#%d%c%c' % (cached_no, c, c))
                else:
                    output.write('#%d!%d%c' % (cached_no, count, c))
            output.write('$')  # write line terminator
            if n == 32:
                n = 1
                output.write('-')  # write sixel line separator
            else:
                n <<= 1


    def __write_body_section(self, output, n):

        data = self.data

        #write body section
        cellheight = self.cellheight
        cellwidth = self.cellwidth
        width = self.cellwidth * self.columns

        if self.palette:
            for y in xrange(cellheight * n, cellheight * (n + 1), 6):
                self._write_colored_sixel(output, data, width, y, -1)
        else:
            startpos = width * cellheight * n
            positive = 255 if self.negate else 0
            for c in xrange(0, self.columns):
                startx = cellwidth * c
                for y in xrange(0, cellheight, 6):
                    if y != 0:
                        output.write("/")
                    for x in range(0, cellwidth):
                        acc = 0
                        for i in xrange(0, 6):
                            acc = acc * 2
                            index = (y + 5 - i) * width + startx + x
                            if data[startpos + index] == positive:
                                acc += 1
                        output.write(chr(acc + 0x3f))
                output.write(";")

    def __write_terminator(self, output):
        # write ST
        output.write(self.ST)  # terminate Device Control String

    def write(self, output):

        if self._use_unicode:
            import codecs
            output = codecs.getwriter("utf-8")(output)

        for n in xrange(0, self.rows):
            self.__write_header(output, 0x40 + n)
            if self._ncolor > 1:
                self._write_sixel_palette(output)
            self.__write_body_section(output, n)
            self.__write_terminator(output)

        output.write("\n")

        if self._use_unicode:
            for dscs in xrange(0, self.rows):
                for c in xrange(0, self.columns):
                    code = 0x100000 | 0x40 + dscs << 8 | 0x21 + c
                    code -= 0x10000
                    c1 = (code >> 10) + 0xd800
                    c2 = (code & 0x3ff) + 0xdc00
                    output.write(unichr(c1) + unichr(c2))
                output.write("\n")
        else:
            for dscs in xrange(0, self.rows):
                output.write("\x1b( %c" % (0x40 + dscs))
                for c in xrange(0, self.columns):
                    output.write(chr(0x21 + c))
                output.write("\x1b(B\n")

class DrcsWriter:

    def __init__(self, f8bit=False):
        self.f8bit = f8bit
        if f8bit:  # 8bit mode
            self.CSI = '\x9b'
        else:
            self.CSI = '\x1b['

    def draw(self, image, columns=62, rows=None,
             negate=False, use_unicode=False,
             output=sys.stdout,
             ncolor=1):
        drcs_converter = DrcsConverter(image, self.f8bit,
                                       columns, rows, negate,
                                       use_unicode,
                                       ncolor=ncolor)
        drcs_converter.write(output)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from drcs import __version__, __license__, __author__
import inspect, os

filename = inspect.getfile(inspect.currentframe())
dirpath = os.path.abspath(os.path.dirname(inspect.getfile(inspect.currentframe())))
long_description = open(os.path.join(dirpath, "README.rst")).read()

import drcs.drcs as drcs
#import drcs.converter as converter

import doctest
dirty = False
for m in [drcs]:
    failure_count, test_count = doctest.testmod(m)
    if failure_count > 0:
        dirty = True
if dirty:
    raise Exception("test failed.")

setup(name                  = 'PyDRCS',
      version               = __version__,
      description           = 'Make DRCS glyphs for DEC VT-series from image files',
      long_description      = long_description,
      py_modules            = ['drcs'],
      eager_resources       = ['drcs/unifont-5.1.20080907.ttf', 'COPYING', 'LICENSE.gnu-unifont.txt'],
      classifiers           = ['Development Status :: 4 - Beta',
                               'Topic :: Terminals',
                               'Environment :: Console',
                               'Intended Audience :: End Users/Desktop',
                               'License :: OSI Approved :: GNU General Public License (GPL)',
                               'Programming Language :: Python'
                               ],
      keywords              = 'drcs terminal sixel image',
      author                = __author__,
      author_email          = 'user@zuse.jp',
      url                   = 'https://github.com/saitoha/PyDRCS',
      license               = __license__,
      packages              = find_packages(exclude=[]),
      zip_safe              = False,
      include_package_data  = True,
      install_requires      = ['Pillow'],
      entry_points          = """
                              [console_scripts]
                              drcsconv = drcs:main
                              """
      )

